@@ -24,16 +24,25 @@ from __future__ import print_function
 
 from tensorflow.contrib.distributions.python.ops import bijectors
 from tensorflow.contrib.distributions.python.ops.binomial import *
+from tensorflow.contrib.distributions.python.ops.cauchy import *
 from tensorflow.contrib.distributions.python.ops.chi2 import *
 from tensorflow.contrib.distributions.python.ops.conditional_distribution import *
 from tensorflow.contrib.distributions.python.ops.conditional_transformed_distribution import *
 from tensorflow.contrib.distributions.python.ops.deterministic import *
+from tensorflow.contrib.distributions.python.ops.distribution_util import fill_triangular
 from tensorflow.contrib.distributions.python.ops.distribution_util import matrix_diag_transform
+from tensorflow.contrib.distributions.python.ops.distribution_util import reduce_weighted_logsumexp
 from tensorflow.contrib.distributions.python.ops.distribution_util import softplus_inverse
+from tensorflow.contrib.distributions.python.ops.distribution_util import tridiag
+from tensorflow.contrib.distributions.python.ops.estimator import *
 from tensorflow.contrib.distributions.python.ops.geometric import *
+from tensorflow.contrib.distributions.python.ops.half_normal import *
+from tensorflow.contrib.distributions.python.ops.independent import *
 from tensorflow.contrib.distributions.python.ops.inverse_gamma import *
 from tensorflow.contrib.distributions.python.ops.logistic import *
 from tensorflow.contrib.distributions.python.ops.mixture import *
+from tensorflow.contrib.distributions.python.ops.mixture_same_family import *
+from tensorflow.contrib.distributions.python.ops.moving_stats import *
 from tensorflow.contrib.distributions.python.ops.mvn_diag import *
 from tensorflow.contrib.distributions.python.ops.mvn_diag_plus_low_rank import *
 from tensorflow.contrib.distributions.python.ops.mvn_full_covariance import *
@@ -42,12 +51,17 @@ from tensorflow.contrib.distributions.python.ops.negative_binomial import *
 from tensorflow.contrib.distributions.python.ops.normal_conjugate_posteriors import *
 from tensorflow.contrib.distributions.python.ops.onehot_categorical import *
 from tensorflow.contrib.distributions.python.ops.poisson import *
+from tensorflow.contrib.distributions.python.ops.poisson_lognormal import *
 from tensorflow.contrib.distributions.python.ops.quantized_distribution import *
 from tensorflow.contrib.distributions.python.ops.relaxed_bernoulli import *
 from tensorflow.contrib.distributions.python.ops.relaxed_onehot_categorical import *
 from tensorflow.contrib.distributions.python.ops.sample_stats import *
+from tensorflow.contrib.distributions.python.ops.sinh_arcsinh import *
+from tensorflow.contrib.distributions.python.ops.test_util import *
+from tensorflow.contrib.distributions.python.ops.vector_diffeomixture import *
 from tensorflow.contrib.distributions.python.ops.vector_exponential_diag import *
 from tensorflow.contrib.distributions.python.ops.vector_laplace_diag import *
+from tensorflow.contrib.distributions.python.ops.vector_sinh_arcsinh_diag import *
 from tensorflow.contrib.distributions.python.ops.wishart import *
 from tensorflow.python.ops.distributions.bernoulli import *
 from tensorflow.python.ops.distributions.beta import *
@@ -71,23 +85,11 @@ from tensorflow.python.util.all_util import remove_undocumented
 
 _allowed_symbols = [
     'bijectors',
+    'Cauchy',
     'ConditionalDistribution',
     'ConditionalTransformedDistribution',
     'FULLY_REPARAMETERIZED',
     'NOT_REPARAMETERIZED',
-    'Affine',
-    'AffineLinearOperator',
-    'Bijector',
-    'Chain',
-    'CholeskyOuterProduct',
-    'Exp',
-    'Identity',
-    'Inline',
-    'Invert',
-    'PowerTransform',
-    'SigmoidCentered',
-    'SoftmaxCentered',
-    'Softplus',
     'ReparameterizationType',
     'Distribution',
     'Binomial',
@@ -106,6 +108,8 @@ _allowed_symbols = [
     'Gamma',
     'GammaWithSoftplusConcentrationRate',
     'Geometric',
+    'HalfNormal',
+    'Independent',
     'InverseGamma',
     'InverseGammaWithSoftplusConcentrationRate',
     'Laplace',
@@ -115,6 +119,8 @@ _allowed_symbols = [
     'Normal',
     'NormalWithSoftplusScale',
     'Poisson',
+    'PoissonLogNormalQuadratureCompound',
+    'SinhArcsinh',
     'StudentT',
     'StudentTWithAbsDfSoftplusScale',
     'Uniform',
@@ -126,23 +132,33 @@ _allowed_symbols = [
     'Dirichlet',
     'DirichletMultinomial',
     'Multinomial',
+    'VectorDiffeomixture',
     'VectorLaplaceDiag',
+    'VectorSinhArcsinhDiag',
     'WishartCholesky',
     'WishartFull',
     'TransformedDistribution',
     'QuantizedDistribution',
     'Mixture',
+    'MixtureSameFamily',
     'ExpRelaxedOneHotCategorical',
     'OneHotCategorical',
     'RelaxedBernoulli',
     'RelaxedOneHotCategorical',
     'kl_divergence',
     'RegisterKL',
+    'fill_triangular',
     'matrix_diag_transform',
+    'reduce_weighted_logsumexp',
+    'softplus_inverse',
+    'tridiag',
     'normal_conjugates_known_scale_posterior',
     'normal_conjugates_known_scale_predictive',
-    'softplus_inverse',
-    'percentile'
+    'percentile',
+    'assign_moving_mean_variance',
+    'assign_log_moving_mean_exp',
+    'moving_mean_variance',
+    'estimator_head_distribution_regression',
 ]
 
 remove_undocumented(__name__, _allowed_symbols)
