@@ -30,6 +30,7 @@ limitations under the License.
 #include "tensorflow/core/framework/op.h"
 #include "tensorflow/core/framework/op_kernel.h"
 #include "tensorflow/core/framework/tensor_types.h"
+#include "tensorflow/core/framework/variant_op_registry.h"
 #include "tensorflow/core/platform/logging.h"
 #include "tensorflow/core/util/bcast.h"
 
@@ -248,6 +249,25 @@ class UnaryOp : public OpKernel {
   }
 };
 
+template <typename Device, VariantUnaryOp OpEnum>
+class UnaryVariantOp : public OpKernel {
+ public:
+  explicit UnaryVariantOp(OpKernelConstruction* ctx) : OpKernel(ctx) {}
+
+  void Compute(OpKernelContext* ctx) override {
+    const Tensor& inp = ctx->input(0);
+    OP_REQUIRES(
+        ctx, TensorShapeUtils::IsScalar(inp.shape()),
+        errors::InvalidArgument("Non-scalar variants are not supported."));
+    const Variant& v = inp.scalar<Variant>()();
+    Variant v_out;
+    OP_REQUIRES_OK(ctx, UnaryOpVariant<Device>(ctx, OpEnum, v, &v_out));
+    Tensor out(cpu_allocator(), DT_VARIANT, TensorShape());
+    out.scalar<Variant>()() = std::move(v_out);
+    ctx->set_output(0, std::move(out));
+  }
+};
+
 namespace functor {
 
 template <typename D, typename Out, typename Rhs>
@@ -285,6 +305,62 @@ struct BinaryFunctor<CPUDevice, Functor, NDIMS, false> {
     Assign(d, out, in.unaryExpr(Unary(scalar.data())));
   }
 
+  void BCast(const CPUDevice& dev,
+             typename TTypes<typename Functor::out_type, NDIMS>::Tensor out,
+             typename TTypes<typename Functor::in_type, NDIMS>::ConstTensor in0,
+             typename Eigen::array<Eigen::DenseIndex, NDIMS> bcast0,
+             typename TTypes<typename Functor::in_type, NDIMS>::ConstTensor in1,
+             typename Eigen::array<Eigen::DenseIndex, NDIMS> bcast1,
+             bool* error) {
+    typename Functor::func func;
+    if (AllOne<NDIMS>(bcast0) && AllOne<NDIMS>(bcast1)) {
+      Assign(dev, out, in0.binaryExpr(in1, func));
+    } else if (AllOne<NDIMS>(bcast0)) {
+      auto rhs = in1.broadcast(bcast1);
+      Assign(dev, out, in0.binaryExpr(rhs, func));
+    } else if (AllOne<NDIMS>(bcast1)) {
+      auto lhs = in0.broadcast(bcast0);
+      Assign(dev, out, lhs.binaryExpr(in1, func));
+    } else {
+      auto lhs = in0.broadcast(bcast0);
+      auto rhs = in1.broadcast(bcast1);
+      Assign(dev, out, lhs.binaryExpr(rhs, func));
+    }
+  }
+};
+
+// Partial specialization of BinaryFunctor<Device=CPUDevice, Functor, 2>
+// for functors with with no error checking.
+template <typename Functor>
+struct BinaryFunctor<CPUDevice, Functor, 2, false> {
+  enum { NDIMS = 2 };
+
+  void operator()(const CPUDevice& d, typename Functor::tout_type out,
+                  typename Functor::tin_type in0,
+                  typename Functor::tin_type in1, bool* error) {
+    Assign(d, out, in0.binaryExpr(in1, typename Functor::func()));
+  }
+
+  void Left(const CPUDevice& d, typename Functor::tout_type out,
+            typename Functor::tscalar_type scalar,
+            typename Functor::tin_type in, bool* error) {
+    typedef typename Functor::out_type Tout;
+    typedef typename Functor::in_type Tin;
+    typedef typename Functor::func Binary;
+    typedef typename Eigen::internal::scalar_left<Tout, Tin, Binary> Unary;
+    Assign(d, out, in.unaryExpr(Unary(scalar.data())));
+  }
+
+  void Right(const CPUDevice& d, typename Functor::tout_type out,
+             typename Functor::tin_type in,
+             typename Functor::tscalar_type scalar, bool* error) {
+    typedef typename Functor::out_type Tout;
+    typedef typename Functor::in_type Tin;
+    typedef typename Functor::func Binary;
+    typedef typename Eigen::internal::scalar_right<Tout, Tin, Binary> Unary;
+    Assign(d, out, in.unaryExpr(Unary(scalar.data())));
+  }
+
 #if !defined(EIGEN_HAS_INDEX_LIST)
   inline Eigen::DSizes<int, 2> NByOne(int n) {
     return Eigen::DSizes<int, 2>(n, 1);
@@ -314,8 +390,7 @@ struct BinaryFunctor<CPUDevice, Functor, NDIMS, false> {
              bool* error) {
     typedef typename Functor::in_type T;
     typename Functor::func func;
-    if ((NDIMS == 2) && Functor::use_bcast_optimization &&
-        use_bcast_optimization<T>::value) {
+    if (Functor::use_bcast_optimization && use_bcast_optimization<T>::value) {
       // Optimize for speed by using Eigen::type2index and avoid
       // .broadcast() when we know its a no-op.
       //
@@ -390,7 +465,7 @@ struct BinaryFunctor<CPUDevice, Functor, NDIMS, false> {
       }
     }
 
-    // Fallback path. Always work and probably slower.
+    // Fallback path. Always works and probably slower.
     auto lhs = in0.broadcast(bcast0);
     auto rhs = in1.broadcast(bcast1);
     Assign(dev, out, lhs.binaryExpr(rhs, func));
@@ -466,6 +541,11 @@ struct ApproximateEqual<CPUDevice, T> {
   REGISTER_KERNEL_BUILDER(Name(N).Device(DEVICE_##D).TypeConstraint<T>("T"), \
                           OP<D##Device, F<T>>);
 
+#define REGISTER_VARIANT(OP, D, N, ENUM)                       \
+  REGISTER_KERNEL_BUILDER(                                     \
+      Name(N).Device(DEVICE_##D).TypeConstraint<Variant>("T"), \
+      OP<D##Device, ENUM>);
+
 // Macros to register kernels for multiple types (T0, T1, etc.)  on
 // device type "D" (CPU or GPU) for operation "N" (e.g., sqrt) using
 // the functor "F" (e.g., functor::sqrt).
@@ -473,7 +553,7 @@ struct ApproximateEqual<CPUDevice, T> {
 #if defined(__ANDROID_TYPES_SLIM__)
 // Note that __ANDROID_TYPES_SLIM__ is also checked in the cwise_ops*.cc files.
 // Normally Android TensorFlow is built with a reduced number of types (float).
-// Override on the command-line "--define ANDROID_TYPES=__ANDROID_TYPES_FULL__"
+// Override on the command-line using "--copt=-D__ANDROID_TYPES_FULL__"
 // to generate a library with full type support with a consequent increase in
 // code size.
 #define REGISTER2(OP, D, N, F, T0, T1) REGISTER(OP, D, N, F, T0)
