@@ -19,6 +19,7 @@ from __future__ import print_function
 
 import numpy as np
 
+from tensorflow.python.eager import context
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
@@ -39,22 +40,25 @@ def _SumGrad(op, grad):
   """Gradient for Sum."""
   # Fast path for when reducing to a scalar and ndims is known: adds only
   # Reshape and Tile ops (and possibly a Shape).
-  if (op.inputs[0].get_shape().ndims is not None and
-      op.inputs[1].op.type == "Const"):
-    rank = op.inputs[0].get_shape().ndims
-    axes = tensor_util.MakeNdarray(op.inputs[1].op.get_attr("value"))
-    if np.array_equal(axes, np.arange(rank)):  # Reduce all dims.
-      grad = array_ops.reshape(grad, [1] * rank)
-      # If shape is not fully defined (but rank is), we use Shape.
-      if op.inputs[0].get_shape().is_fully_defined():
-        input_shape = op.inputs[0].get_shape().as_list()
-      else:
-        input_shape = array_ops.shape(op.inputs[0])
-      return [array_ops.tile(grad, input_shape), None]
+  if op.inputs[0].get_shape().ndims is not None:
+    axes = tensor_util.constant_value(op.inputs[1])
+    if axes is not None:
+      rank = op.inputs[0].get_shape().ndims
+      if np.array_equal(axes, np.arange(rank)):  # Reduce all dims.
+        grad = array_ops.reshape(grad, [1] * rank)
+        # If shape is not fully defined (but rank is), we use Shape.
+        if op.inputs[0].get_shape().is_fully_defined():
+          input_shape = op.inputs[0].get_shape().as_list()
+        else:
+          input_shape = array_ops.shape(op.inputs[0])
+        return [array_ops.tile(grad, input_shape), None]
 
   input_shape = array_ops.shape(op.inputs[0])
-  output_shape_kept_dims = math_ops.reduced_shape(input_shape, op.inputs[1])
-  tile_scaling = _safe_shape_div(input_shape, output_shape_kept_dims)
+  # TODO(apassos) remove this once device placement for eager ops makes more
+  # sense.
+  with ops.colocate_with(input_shape):
+    output_shape_kept_dims = math_ops.reduced_shape(input_shape, op.inputs[1])
+    tile_scaling = _safe_shape_div(input_shape, output_shape_kept_dims)
   grad = array_ops.reshape(grad, output_shape_kept_dims)
   return [array_ops.tile(grad, tile_scaling), None]
 
@@ -92,10 +96,16 @@ def _MinGrad(op, grad):
 def _MeanGrad(op, grad):
   """Gradient for Mean."""
   sum_grad = _SumGrad(op, grad)[0]
-  input_shape = array_ops.shape(op.inputs[0])
-  output_shape = array_ops.shape(op.outputs[0])
-  factor = _safe_shape_div(
-      math_ops.reduce_prod(input_shape), math_ops.reduce_prod(output_shape))
+  input_size = op.inputs[0].get_shape().num_elements()
+  output_size = op.outputs[0].get_shape().num_elements()
+  if input_size is not None and output_size is not None:
+    factor = input_size // max(output_size, 1)
+    factor = constant_op.constant(factor, dtype=sum_grad.dtype)
+  else:
+    input_shape = array_ops.shape(op.inputs[0])
+    output_shape = array_ops.shape(op.outputs[0])
+    factor = _safe_shape_div(
+        math_ops.reduce_prod(input_shape), math_ops.reduce_prod(output_shape))
   return sum_grad / math_ops.cast(factor, sum_grad.dtype), None
 
 
@@ -174,6 +184,15 @@ def _SparseSegmentSumGrad(op, grad):
           None)
 
 
+@ops.RegisterGradient("SparseSegmentSumWithNumSegments")
+def _SparseSegmentSumWithNumSegmentsGrad(op, grad):
+  """Gradient for SparseSegmentSumWithNumSegments."""
+  input_rows = array_ops.shape(op.inputs[0])[0]
+  return (math_ops.unsorted_segment_sum(
+      array_ops.gather(grad, op.inputs[2]), op.inputs[1], input_rows), None,
+          None, None)
+
+
 @ops.RegisterGradient("SparseSegmentMean")
 def _SparseSegmentMeanGrad(op, grad):
   """Gradient for SparseSegmentMean."""
@@ -182,6 +201,14 @@ def _SparseSegmentMeanGrad(op, grad):
                                             dim0), None, None)
 
 
+@ops.RegisterGradient("SparseSegmentMeanWithNumSegments")
+def _SparseSegmentMeanWithNumSegmentsGrad(op, grad):
+  """Gradient for SparseSegmentMeanWithNumSegments."""
+  dim0 = array_ops.shape(op.inputs[0])[0]
+  return (math_ops.sparse_segment_mean_grad(grad, op.inputs[1], op.inputs[2],
+                                            dim0), None, None, None)
+
+
 @ops.RegisterGradient("SparseSegmentSqrtN")
 def _SparseSegmentSqrtNGrad(op, grad):
   """Gradient for SparseSegmentSqrtN."""
@@ -190,6 +217,14 @@ def _SparseSegmentSqrtNGrad(op, grad):
                                               dim0), None, None)
 
 
+@ops.RegisterGradient("SparseSegmentSqrtNWithNumSegments")
+def _SparseSegmentSqrtNWithNumSegmentsGrad(op, grad):
+  """Gradient for SparseSegmentSqrtNWithNumSegmnets."""
+  dim0 = array_ops.shape(op.inputs[0])[0]
+  return (math_ops.sparse_segment_sqrt_n_grad(grad, op.inputs[1], op.inputs[2],
+                                              dim0), None, None, None)
+
+
 def _SegmentMinOrMaxGrad(op, grad, is_sorted):
   """Gradient for SegmentMin and (unsorted) SegmentMax. They share similar code."""
   zeros = array_ops.zeros(array_ops.shape(op.inputs[0]),
@@ -202,8 +237,8 @@ def _SegmentMinOrMaxGrad(op, grad, is_sorted):
     num_selected = math_ops.segment_sum(math_ops.cast(is_selected, grad.dtype),
                                         op.inputs[1])
   else:
-    num_selected = math_ops.unsorted_segment_sum(math_ops.cast(is_selected, grad.dtype),
-                                                 op.inputs[1], op.inputs[2])
+    num_selected = math_ops.unsorted_segment_sum(
+        math_ops.cast(is_selected, grad.dtype), op.inputs[1], op.inputs[2])
 
   # Compute the gradient for each segment. The gradient for the ith segment is
   # divided evenly among the selected elements in that segment.
@@ -271,7 +306,7 @@ def _ReciprocalGrad(op, grad):
 def _InvGradGrad(op, grad):
   b = op.inputs[1]
   # op.output[0]: y = -b * conj(a)^2
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     ca = math_ops.conj(op.inputs[0])
     cg = math_ops.conj(grad)
     # pylint: disable=protected-access
@@ -282,7 +317,7 @@ def _InvGradGrad(op, grad):
 def _ReciprocalGradGrad(op, grad):
   b = op.inputs[1]
   # op.output[0]: y = -b * conj(a)^2
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     ca = math_ops.conj(op.inputs[0])
     cg = math_ops.conj(grad)
     # pylint: disable=protected-access
@@ -293,7 +328,7 @@ def _ReciprocalGradGrad(op, grad):
 def _SquareGrad(op, grad):
   x = op.inputs[0]
   # Added control dependencies to prevent 2*x from being computed too early.
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * (2.0 * x)
 
@@ -301,14 +336,16 @@ def _SquareGrad(op, grad):
 @ops.RegisterGradient("Sqrt")
 def _SqrtGrad(op, grad):
   y = op.outputs[0]  # y = x^(1/2)
+  # pylint: disable=protected-access
   return gen_math_ops._sqrt_grad(y, grad)
+  # pylint: enable=protected-access
 
 
 @ops.RegisterGradient("SqrtGrad")
 def _SqrtGradGrad(op, grad):
   a = op.inputs[0]
   y = op.outputs[0]  # y = 0.5 * b / conj(a)
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     ga = grad / a
     return -math_ops.conj(ga) * y, 0.5 * ga
 
@@ -317,7 +354,9 @@ def _SqrtGradGrad(op, grad):
 def _RsqrtGrad(op, grad):
   """Returns -0.5 * grad * conj(y)^3."""
   y = op.outputs[0]  # y = x^(-1/2)
+  # pylint: disable=protected-access
   return gen_math_ops._rsqrt_grad(y, grad)
+  # pylint: enable=protected-access
 
 
 @ops.RegisterGradient("RsqrtGrad")
@@ -325,7 +364,7 @@ def _RsqrtGradGrad(op, grad):
   """Returns backprop gradient for f(a,b) = -0.5 * b * conj(a)^3."""
   a = op.inputs[0]  # a = x^{-1/2}
   b = op.inputs[1]  # backprop gradient for a
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     ca = math_ops.conj(a)
     cg = math_ops.conj(grad)
     grad_a = -1.5 * cg * b * math_ops.square(ca)
@@ -338,7 +377,7 @@ def _RsqrtGradGrad(op, grad):
 def _ExpGrad(op, grad):
   """Returns grad * exp(x)."""
   y = op.outputs[0]  # y = e^x
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     y = math_ops.conj(y)
     return grad * y
 
@@ -347,7 +386,7 @@ def _ExpGrad(op, grad):
 def _Expm1Grad(op, grad):
   """Returns grad * exp(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     y = math_ops.exp(x)
     return grad * y
@@ -357,7 +396,7 @@ def _Expm1Grad(op, grad):
 def _LogGrad(op, grad):
   """Returns grad * (1/x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.reciprocal(x)
 
@@ -366,7 +405,7 @@ def _LogGrad(op, grad):
 def _Log1pGrad(op, grad):
   """Returns grad * (1/(1 + x))."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.reciprocal(1 + x)
 
@@ -375,7 +414,7 @@ def _Log1pGrad(op, grad):
 def _SinhGrad(op, grad):
   """Returns grad * cosh(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.cosh(x)
 
@@ -384,7 +423,7 @@ def _SinhGrad(op, grad):
 def _CoshGrad(op, grad):
   """Returns grad * sinh(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.sinh(x)
 
@@ -393,7 +432,7 @@ def _CoshGrad(op, grad):
 def _TanhGrad(op, grad):
   """Returns grad * (1 - tanh(x) * tanh(x))."""
   y = op.outputs[0]  # y = tanh(x)
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     y = math_ops.conj(y)
     # pylint: disable=protected-access
     return gen_math_ops._tanh_grad(y, grad)
@@ -403,7 +442,7 @@ def _TanhGrad(op, grad):
 def _AsinhGrad(op, grad):
   """Returns grad * 1/cosh(y)."""
   y = op.outputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     y = math_ops.conj(y)
     return grad / math_ops.cosh(y)
 
@@ -412,7 +451,7 @@ def _AsinhGrad(op, grad):
 def _AcoshGrad(op, grad):
   """Returns grad * 1/sinh(y)."""
   y = op.outputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     y = math_ops.conj(y)
     return grad / math_ops.sinh(y)
 
@@ -421,7 +460,7 @@ def _AcoshGrad(op, grad):
 def _AtanhGrad(op, grad):
   """Returns grad * 1/ (1 - x^2)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
@@ -431,7 +470,7 @@ def _AtanhGrad(op, grad):
 
 @ops.RegisterGradient("TanhGrad")
 def _TanhGradGrad(op, grad):
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     a = math_ops.conj(op.inputs[0])
     b = math_ops.conj(op.inputs[1])
     # pylint: disable=protected-access
@@ -443,7 +482,7 @@ def _ErfGrad(op, grad):
   """Returns grad * 2/sqrt(pi) * exp(-x**2)."""
   x = op.inputs[0]
   two_over_root_pi = constant_op.constant(2 / np.sqrt(np.pi), dtype=grad.dtype)
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * two_over_root_pi * math_ops.exp(-math_ops.square(x))
 
@@ -454,7 +493,7 @@ def _ErfcGrad(op, grad):
   x = op.inputs[0]
   minus_two_over_root_pi = constant_op.constant(
       -2 / np.sqrt(np.pi), dtype=grad.dtype)
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * minus_two_over_root_pi * math_ops.exp(-math_ops.square(x))
 
@@ -463,7 +502,7 @@ def _ErfcGrad(op, grad):
 def _LgammaGrad(op, grad):
   """Returns grad * digamma(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.digamma(x)
 
@@ -472,7 +511,7 @@ def _LgammaGrad(op, grad):
 def _DigammaGrad(op, grad):
   """Compute gradient of the digamma function with respect to its argument."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.polygamma(array_ops.constant(1, dtype=x.dtype), x)
 
@@ -485,7 +524,9 @@ def _IgammaGrad(op, grad):
   x = op.inputs[1]
   sa = array_ops.shape(a)
   sx = array_ops.shape(x)
+  # pylint: disable=protected-access
   unused_ra, rx = gen_array_ops._broadcast_gradient_args(sa, sx)
+  # pylint: enable=protected-access
 
   # Perform operations in log space before summing, because Gamma(a)
   # and Gamma'(a) can grow large.
@@ -538,9 +579,11 @@ def _ZetaGrad(op, grad):
   # Broadcast gradients
   sx = array_ops.shape(x)
   sq = array_ops.shape(q)
+  # pylint: disable=protected-access
   unused_rx, rq = gen_array_ops._broadcast_gradient_args(sx, sq)
+  # pylint: enable=protected-access
   # Evaluate gradient
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     q = math_ops.conj(q)
     partial_q = -x * math_ops.zeta(x + 1, q)
@@ -558,9 +601,11 @@ def _PolygammaGrad(op, grad):
   # Broadcast gradients
   sn = array_ops.shape(n)
   sx = array_ops.shape(x)
+  # pylint: disable=protected-access
   unused_rn, rx = gen_array_ops._broadcast_gradient_args(sn, sx)
+  # pylint: enable=protected-access
   # Evaluate gradient
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     n = math_ops.conj(n)
     x = math_ops.conj(x)
     partial_x = math_ops.polygamma(n + 1, x)
@@ -573,7 +618,7 @@ def _PolygammaGrad(op, grad):
 def _SigmoidGrad(op, grad):
   """Returns grad * sigmoid(x) * (1 - sigmoid(x))."""
   y = op.outputs[0]  # y = sigmoid(x)
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     y = math_ops.conj(y)
     # pylint: disable=protected-access
     return gen_math_ops._sigmoid_grad(y, grad)
@@ -581,7 +626,7 @@ def _SigmoidGrad(op, grad):
 
 @ops.RegisterGradient("SigmoidGrad")
 def _SigmoidGradGrad(op, grad):
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     a = math_ops.conj(op.inputs[0])
     b = math_ops.conj(op.inputs[1])
     gb = grad * b
@@ -600,7 +645,7 @@ def _SignGrad(op, _):
 def _SinGrad(op, grad):
   """Returns grad * cos(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return grad * math_ops.cos(x)
 
@@ -609,7 +654,7 @@ def _SinGrad(op, grad):
 def _CosGrad(op, grad):
   """Returns grad * -sin(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     return -grad * math_ops.sin(x)
 
@@ -618,7 +663,7 @@ def _CosGrad(op, grad):
 def _TanGrad(op, grad):
   """Returns grad * 1/sec^2(x)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     secx = math_ops.reciprocal(math_ops.cos(x))
     secx2 = math_ops.square(secx)
@@ -629,7 +674,7 @@ def _TanGrad(op, grad):
 def _AsinGrad(op, grad):
   """Returns grad * 1/sqrt(1-x^2)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
@@ -642,7 +687,7 @@ def _AsinGrad(op, grad):
 def _AcosGrad(op, grad):
   """Returns grad * -1/sqrt(1-x^2)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
@@ -655,7 +700,7 @@ def _AcosGrad(op, grad):
 def _AtanGrad(op, grad):
   """Returns grad * 1/ (1 + x^2)."""
   x = op.inputs[0]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     x = math_ops.conj(x)
     x2 = math_ops.square(x)
     one = constant_op.constant(1, dtype=grad.dtype)
@@ -668,7 +713,7 @@ def _Atan2Grad(op, grad):
   """Returns grad * x / (x^2 + y^2), grad * -y / (x^2 + y^2)."""
   y = op.inputs[0]
   x = op.inputs[1]
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     grad_inv = grad / (math_ops.square(x) + math_ops.square(y))
     return x * grad_inv, -y * grad_inv
 
@@ -680,13 +725,31 @@ def _AddNGrad(op, grad):
   return [grad] * len(op.inputs)
 
 
+def _ShapesFullySpecifiedAndEqual(x, y, grad):
+  # pylint: disable=protected-access
+  x_shape = x._shape_tuple()
+  y_shape = y._shape_tuple()
+  grad_shape = grad._shape_tuple()
+  # pylint: enable=protected-access
+  return (x_shape == y_shape and
+          x_shape == grad_shape and
+          x_shape is not None and
+          None not in x_shape)
+
+
 @ops.RegisterGradient("Add")
 def _AddGrad(op, grad):
+  """Gradient for Add."""
   x = op.inputs[0]
   y = op.inputs[1]
+  if (isinstance(grad, ops.Tensor) and
+      _ShapesFullySpecifiedAndEqual(x, y, grad)):
+    return grad, grad
   sx = array_ops.shape(x)
   sy = array_ops.shape(y)
+  # pylint: disable=protected-access
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
   return (array_ops.reshape(math_ops.reduce_sum(grad, rx), sx),
           array_ops.reshape(math_ops.reduce_sum(grad, ry), sy))
 
@@ -697,7 +760,9 @@ def _SubGrad(op, grad):
   y = op.inputs[1]
   sx = array_ops.shape(x)
   sy = array_ops.shape(y)
+  # pylint: disable=protected-access
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
   return (array_ops.reshape(math_ops.reduce_sum(grad, rx), sx),
           array_ops.reshape(-math_ops.reduce_sum(grad, ry), sy))
 
@@ -707,10 +772,16 @@ def _MulGrad(op, grad):
   """The gradient of scalar multiplication."""
   x = op.inputs[0]
   y = op.inputs[1]
+  # pylint: disable=protected-access
+  if (isinstance(grad, ops.Tensor) and
+      _ShapesFullySpecifiedAndEqual(x, y, grad) and
+      grad.dtype in (dtypes.int32, dtypes.float32)):
+    return gen_math_ops._mul(grad, y), gen_math_ops._mul(grad, x)
   assert x.dtype.base_dtype == y.dtype.base_dtype, (x.dtype, " vs. ", y.dtype)
   sx = array_ops.shape(x)
   sy = array_ops.shape(y)
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
   x = math_ops.conj(x)
   y = math_ops.conj(y)
   return (array_ops.reshape(math_ops.reduce_sum(grad * y, rx), sx),
@@ -741,6 +812,24 @@ def _FloorDivGrad(_, unused_grad):
   return None, None
 
 
+@ops.RegisterGradient("FloorMod")
+def _FloorModGrad(op, grad):
+  """Returns grad * (1, -floor(x/y))."""
+  x = math_ops.conj(op.inputs[0])
+  y = math_ops.conj(op.inputs[1])
+
+  sx = array_ops.shape(x)
+  sy = array_ops.shape(y)
+  # pylint: disable=protected-access
+  rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
+  # pylint: enable=protected-access
+  floor_xy = math_ops.floor_div(x, y)
+  gx = array_ops.reshape(math_ops.reduce_sum(grad, rx), sx)
+  gy = array_ops.reshape(
+      math_ops.reduce_sum(grad * math_ops.negative(floor_xy), ry), sy)
+  return gx, gy
+
+
 @ops.RegisterGradient("TruncateDiv")
 def _TruncateDivGrad(_, unused_grad):
   return None, None
@@ -803,7 +892,7 @@ def _MaximumMinimumGrad(op, grad, selector_op):
   xmask = selector_op(x, y)
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
   xgrad = array_ops.where(xmask, grad, zeros)
-  ygrad = array_ops.where(math_ops.logical_not(xmask), grad, zeros)
+  ygrad = array_ops.where(xmask, zeros, grad)
   gx = array_ops.reshape(math_ops.reduce_sum(xgrad, rx), sx)
   gy = array_ops.reshape(math_ops.reduce_sum(ygrad, ry), sy)
   return (gx, gy)
@@ -831,8 +920,7 @@ def _SquaredDifferenceGrad(op, grad):
   # pylint: disable=protected-access
   rx, ry = gen_array_ops._broadcast_gradient_args(sx, sy)
   # pylint: enable=protected-access
-  # .op works with Tensors or IndexedSlices
-  with ops.control_dependencies([grad.op]):
+  with ops.control_dependencies([grad]):
     # The parens ensure that if grad is IndexedSlices, it'll get multiplied by
     # Tensor (not a number like 2.0) which causes it to convert to Tensor.
     x_grad = math_ops.scalar_mul(2.0, grad) * (x - y)
@@ -895,7 +983,7 @@ def _SparseMatMulGrad(op, grad):
       op.inputs[0]: op.get_attr("a_is_sparse"),
       op.inputs[1]: op.get_attr("b_is_sparse"),
       # Use heuristic to figure out if grad might be sparse
-      grad: (grad.op.type == "ReluGrad")
+      grad: context.in_graph_mode() and (grad.op.type == "ReluGrad")
   }
 
   def _SparseMatMul(t1, t2, out_dtype, transpose_a=False, transpose_b=False):
@@ -1015,6 +1103,19 @@ def _ImagGrad(_, grad):
   return math_ops.complex(zero, grad)
 
 
+@ops.RegisterGradient("Angle")
+def _AngleGrad(op, grad):
+  """Returns -grad / (Im(x) + iRe(x))"""
+  x = op.inputs[0]
+  with ops.control_dependencies([grad]):
+    re = math_ops.real(x)
+    im = math_ops.imag(x)
+    z = math_ops.reciprocal(math_ops.complex(im, re))
+    zero = constant_op.constant(0, dtype=grad.dtype)
+    complex_grad = math_ops.complex(grad, zero)
+    return -complex_grad * z
+
+
 @ops.RegisterGradient("Conj")
 def _ConjGrad(_, grad):
   """Returns the complex conjugate of grad."""
